# frozen_string_literal: true

RSpec.describe "Predicates: True" do
  context "with key" do
    subject(:schema) do
      Dry::Schema.Params do
        required(:foo).value(:true)
      end
    end

    context "with valid input (1)" do
      let(:input) { {"foo" => "1"} }

      it "is successful" do
        expect(result).to be_successful
      end
    end

    context "with valid input (true)" do
      let(:input) { {"foo" => "true"} }

      it "is successful" do
        expect(result).to be_successful
      end
    end

    context "with missing input" do
      let(:input) { {} }

      it "is not successful" do
        expect(result).to be_failing ["is missing", "must be true"]
      end
    end

    context "with nil input" do
      let(:input) { {"foo" => nil} }

      it "is not successful" do
        expect(result).to be_failing ["must be true"]
      end
    end

    context "with blank input" do
      let(:input) { {"foo" => ""} }

      it "is not successful" do
        expect(result).to be_failing ["must be true"]
      end
    end

    context "with invalid input" do
      let(:input) { {"foo" => []} }

      it "is not successful" do
        expect(result).to be_failing ["must be true"]
      end
    end
  end

  context "with optional" do
    subject(:schema) do
      Dry::Schema.Params do
        optional(:foo).value(:true)
      end
    end

    context "with valid input (1)" do
      let(:input) { {"foo" => "1"} }

      it "is successful" do
        expect(result).to be_successful
      end
    end

    context "with valid input (true)" do
      let(:input) { {"foo" => "true"} }

      it "is successful" do
        expect(result).to be_successful
      end
    end

    context "with missing input" do
      let(:input) { {} }

      it "is successful" do
        expect(result).to be_successful
      end
    end

    context "with nil input" do
      let(:input) { {"foo" => nil} }

      it "is not successful" do
        expect(result).to be_failing ["must be true"]
      end
    end

    context "with blank input" do
      let(:input) { {"foo" => ""} }

      it "is not successful" do
        expect(result).to be_failing ["must be true"]
      end
    end

    context "with invalid input" do
      let(:input) { {"foo" => []} }

      it "is not successful" do
        expect(result).to be_failing ["must be true"]
      end
    end
  end

  context "as macro" do
    context "with required" do
      context "with value" do
        subject(:schema) do
          Dry::Schema.Params do
            required(:foo).value(:true)
          end
        end

        context "with valid input (1)" do
          let(:input) { {"foo" => "1"} }

          it "is successful" do
            expect(result).to be_successful
          end
        end

        context "with valid input (true)" do
          let(:input) { {"foo" => "true"} }

          it "is successful" do
            expect(result).to be_successful
          end
        end

        context "with missing input" do
          let(:input) { {} }

          it "is not successful" do
            expect(result).to be_failing ["is missing", "must be true"]
          end
        end

        context "with nil input" do
          let(:input) { {"foo" => nil} }

          it "is not successful" do
            expect(result).to be_failing ["must be true"]
          end
        end

        context "with blank input" do
          let(:input) { {"foo" => ""} }

          it "is not successful" do
            expect(result).to be_failing ["must be true"]
          end
        end

        context "with invalid input" do
          let(:input) { {"foo" => []} }

          it "is not successful" do
            expect(result).to be_failing ["must be true"]
          end
        end
      end

      context "with filled" do
        subject(:schema) do
          Dry::Schema.Params do
            required(:foo).filled(:true)
          end
        end

        context "with valid input (1)" do
          let(:input) { {"foo" => "1"} }

          it "is successful" do
            expect(result).to be_successful
          end
        end

        context "with valid input (true)" do
          let(:input) { {"foo" => "true"} }

          it "is successful" do
            expect(result).to be_successful
          end
        end

        context "with missing input" do
          let(:input) { {} }

          it "is not successful" do
            expect(result).to be_failing ["is missing", "must be true"]
          end
        end

        context "with nil input" do
          let(:input) { {"foo" => nil} }

          it "is not successful" do
            expect(result).to be_failing ["must be filled", "must be true"]
          end
        end

        context "with blank input" do
          let(:input) { {"foo" => ""} }

          it "is not successful" do
            expect(result).to be_failing ["must be filled", "must be true"]
          end
        end

        context "with invalid input" do
          let(:input) { {"foo" => []} }

          it "is not successful" do
            expect(result).to be_failing ["must be filled", "must be true"]
          end
        end
      end

      context "with maybe" do
        subject(:schema) do
          Dry::Schema.Params do
            required(:foo).maybe(:true)
          end
        end

        context "with valid input (1)" do
          let(:input) { {"foo" => "1"} }

          it "is successful" do
            expect(result).to be_successful
          end
        end

        context "with valid input (true)" do
          let(:input) { {"foo" => "true"} }

          it "is successful" do
            expect(result).to be_successful
          end
        end

        context "with missing input" do
          let(:input) { {} }

          it "is not successful" do
            expect(result).to be_failing ["is missing", "must be true"]
          end
        end

        context "with nil input" do
          let(:input) { {"foo" => nil} }

          it "is successful" do
            expect(result).to be_successful
          end
        end

        context "with blank input" do
          let(:input) { {"foo" => ""} }

          it "is successful" do
            expect(result).to be_successful
          end
        end

        context "with invalid input" do
          let(:input) { {"foo" => []} }

          it "is not successful" do
            expect(result).to be_failing ["must be true"]
          end
        end
      end
    end

    context "with optional" do
      context "with value" do
        subject(:schema) do
          Dry::Schema.Params do
            optional(:foo).value(:true)
          end
        end

        context "with valid input (1)" do
          let(:input) { {"foo" => "1"} }

          it "is successful" do
            expect(result).to be_successful
          end
        end

        context "with valid input (true)" do
          let(:input) { {"foo" => "true"} }

          it "is successful" do
            expect(result).to be_successful
          end
        end

        context "with missing input" do
          let(:input) { {} }

          it "is successful" do
            expect(result).to be_successful
          end
        end

        context "with nil input" do
          let(:input) { {"foo" => nil} }

          it "is not successful" do
            expect(result).to be_failing ["must be true"]
          end
        end

        context "with blank input" do
          let(:input) { {"foo" => ""} }

          it "is not successful" do
            expect(result).to be_failing ["must be true"]
          end
        end

        context "with invalid input" do
          let(:input) { {"foo" => []} }

          it "is not successful" do
            expect(result).to be_failing ["must be true"]
          end
        end
      end

      context "with filled" do
        subject(:schema) do
          Dry::Schema.Params do
            optional(:foo).filled(:true)
          end
        end

        context "with valid input (1)" do
          let(:input) { {"foo" => "1"} }

          it "is successful" do
            expect(result).to be_successful
          end
        end

        context "with valid input (true)" do
          let(:input) { {"foo" => "true"} }

          it "is successful" do
            expect(result).to be_successful
          end
        end

        context "with missing input" do
          let(:input) { {} }

          it "is successful" do
            expect(result).to be_successful
          end
        end

        context "with nil input" do
          let(:input) { {"foo" => nil} }

          it "is not successful" do
            expect(result).to be_failing ["must be filled", "must be true"]
          end
        end

        context "with blank input" do
          let(:input) { {"foo" => ""} }

          it "is not successful" do
            expect(result).to be_failing ["must be filled", "must be true"]
          end
        end

        context "with invalid input" do
          let(:input) { {"foo" => []} }

          it "is not successful" do
            expect(result).to be_failing ["must be filled", "must be true"]
          end
        end
      end

      context "with maybe" do
        subject(:schema) do
          Dry::Schema.Params do
            optional(:foo).maybe(:true)
          end
        end

        context "with valid input (1)" do
          let(:input) { {"foo" => "1"} }

          it "is successful" do
            expect(result).to be_successful
          end
        end

        context "with valid input (true)" do
          let(:input) { {"foo" => "true"} }

          it "is successful" do
            expect(result).to be_successful
          end
        end

        context "with missing input" do
          let(:input) { {} }

          it "is successful" do
            expect(result).to be_successful
          end
        end

        context "with nil input" do
          let(:input) { {"foo" => nil} }

          it "is successful" do
            expect(result).to be_successful
          end
        end

        context "with blank input" do
          let(:input) { {"foo" => ""} }

          it "is successful" do
            expect(result).to be_successful
          end
        end

        context "with invalid input" do
          let(:input) { {"foo" => []} }

          it "is not successful" do
            expect(result).to be_failing ["must be true"]
          end
        end
      end
    end
  end
end
